import android.Manifest
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothDevice
import android.bluetooth.BluetoothSocket
import android.content.Context
import android.content.pm.PackageManager
import android.os.Build
import android.util.Log
import android.widget.Toast
import androidx.annotation.RequiresApi
import androidx.core.app.ActivityCompat
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import java.io.IOException
import java.util.*

object BluetoothManager {
    var bluetoothSocket: BluetoothSocket? = null
    var isConnected: Boolean = false

    @RequiresApi(Build.VERSION_CODES.GINGERBREAD_MR1)
    suspend fun connectToDevice(context: Context, device: BluetoothDevice, uuid: UUID): Boolean {
        return withContext(Dispatchers.IO) {
            try {
                // Check for Bluetooth permissions (handle this in the calling Activity)
                if (ActivityCompat.checkSelfPermission(
                        context,
                        Manifest.permission.BLUETOOTH_CONNECT
                    ) != PackageManager.PERMISSION_GRANTED
                ) {
                    Log.e("BluetoothManager", "Missing BLUETOOTH_CONNECT permission")
                    return@withContext false // return false
                }

                if (bluetoothSocket == null || !isConnected) {
                    bluetoothSocket = device.createInsecureRfcommSocketToServiceRecord(uuid)
                    BluetoothAdapter.getDefaultAdapter()?.cancelDiscovery()
                    bluetoothSocket?.connect()
                    isConnected = true
                    Toast.makeText(context, "Connected to device", Toast.LENGTH_SHORT).show()
                    Log.e("BluetoothManager", "Connected to device: $device")
                }
                true
            } catch (e: IOException) {
                e.printStackTrace()
                isConnected = false
                Log.e("BluetoothManager", "Connection failed: ${e.message}")
                false
            }
        }
    }

    suspend fun disconnect(): Boolean {
        return withContext(Dispatchers.IO) {
            try {
                bluetoothSocket?.close()
                bluetoothSocket = null
                isConnected = false
                true
            } catch (e: IOException) {
                e.printStackTrace()
                false
            }
        }
    }

    suspend fun sendCommand(command: String): Boolean {
        return withContext(Dispatchers.IO) {
            try {
                bluetoothSocket?.outputStream?.write(command.toByteArray())
                true
            } catch (e: IOException) {
                e.printStackTrace()
                false
            }
        }
    }
}
