package com.kedda.carApp
import BluetoothManager
import android.app.Activity
import android.Manifest
import android.annotation.SuppressLint
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothDevice
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.widget.Toast
import android.util.Log

import androidx.activity.enableEdgeToEdge
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import android.widget.ArrayAdapter
import android.widget.Button
import android.widget.ListView
import androidx.annotation.RequiresApi

import com.kedda.carApp.ControlActivity.Companion.m_address
import com.kedda.carApp.ControlActivity.Companion.m_myUUID
import androidx.appcompat.widget.AppCompatImageButton
import androidx.lifecycle.lifecycleScope
import kotlinx.coroutines.launch
open class MainActivity : AppCompatActivity() {

    private var m_bluetoothAdapter: BluetoothAdapter? = null
    private lateinit var m_pairedDevices: Set<BluetoothDevice>
    private val REQUEST_ENABLE_BLUETOOTH = 1
    private val REQUEST_PERMISSIONS = 2


    companion object {
        const val EXTRA_ADDRESS: String = "ESP32-BT-CARRR"
    }

    @RequiresApi(Build.VERSION_CODES.S)
    @SuppressLint("MissingInflatedId", "MissingPermission")
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        enableEdgeToEdge()
        setContentView(R.layout.activity_main)
        m_bluetoothAdapter = BluetoothAdapter.getDefaultAdapter()

        if (m_bluetoothAdapter == null) {
            //Toast.makeText(this, "Device does not support Bluetooth", Toast.LENGTH_SHORT).show()
            val intent = Intent(this, MainActivity::class.java)
            intent.putExtra(EXTRA_ADDRESS, m_address)
            startActivity(intent)
            finish()
            return
        }
        checkPermissions()

        if (!m_bluetoothAdapter!!.isEnabled) {
            val enableBluetoothIntent = Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE)
            startActivityForResult(enableBluetoothIntent, REQUEST_ENABLE_BLUETOOTH)
        } else {
            pairedDeviceList()
        }

        val selectDeviceRefresh = findViewById<Button>(R.id.select_device_refresh)
        selectDeviceRefresh.setOnClickListener { pairedDeviceList() }
    }

    @RequiresApi(Build.VERSION_CODES.S)
    private fun checkPermissions() {
        val permissionsNeeded = mutableListOf(
            Manifest.permission.BLUETOOTH,
            Manifest.permission.BLUETOOTH_SCAN,
            Manifest.permission.BLUETOOTH_CONNECT,
            Manifest.permission.ACCESS_FINE_LOCATION,
            Manifest.permission.ACCESS_COARSE_LOCATION
        )

        val permissionsToRequest = permissionsNeeded.filter {
            ActivityCompat.checkSelfPermission(this, it) != PackageManager.PERMISSION_GRANTED
        }

        if (permissionsToRequest.isNotEmpty()) {
            ActivityCompat.requestPermissions(this, permissionsToRequest.toTypedArray(), REQUEST_PERMISSIONS)
        } else {
            pairedDeviceList()
            //Toast.makeText(this, "Permissions already granted", Toast.LENGTH_SHORT).show()
        }
    }


    // Inside pairedDeviceList function
    private fun pairedDeviceList() {
        m_pairedDevices = m_bluetoothAdapter!!.bondedDevices
        val list = ArrayList<String>()

        if (m_pairedDevices.isNotEmpty()) {
            for (device in m_pairedDevices) {
                if (ActivityCompat.checkSelfPermission(
                        this,
                        Manifest.permission.BLUETOOTH_CONNECT
                    ) != PackageManager.PERMISSION_GRANTED
                ) {
                    return
                }
                list.add("${device.name} - ${device.address}")
                Log.i("device", "Device found: ${device.name} - ${device.address}")
            }
        } else {
           // Toast.makeText(this, "No paired devices", Toast.LENGTH_SHORT).show()
        }
        val adapter = ArrayAdapter(this, android.R.layout.simple_list_item_1, list)
        val deviceListView = findViewById<ListView>(R.id.select_device_list)
        deviceListView.adapter = adapter

        // Set an on item click listener for the list
        deviceListView.setOnItemClickListener { _, _, position, _ ->
            val selectedDevice = m_pairedDevices.elementAt(position) // Get BluetoothDevice object
            m_address = selectedDevice.address // Initialize m_address with the selected device's address
            lifecycleScope.launch {
                BluetoothManager.connectToDevice(this@MainActivity, selectedDevice, m_myUUID)
                if(!BluetoothManager.isConnected) {
                    Log.e("MainActivity", "Connection failed in MainActivity")
                    Toast.makeText(this@MainActivity, "Bluetooth Connection Failed", Toast.LENGTH_SHORT).show()
                    val intent = Intent(this@MainActivity, MainActivity::class.java)
                    intent.putExtra(EXTRA_ADDRESS, m_address)
                    startActivity(intent)
                    finish()
                }
                val intent = Intent(this@MainActivity, ControlActivity::class.java)
                intent.putExtra(EXTRA_ADDRESS, m_address)
                startActivity(intent)
                finish()
            }


        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == REQUEST_ENABLE_BLUETOOTH) {
            if (resultCode == Activity.RESULT_OK) {
                //.makeText(this, "Bluetooth has been enabled", Toast.LENGTH_SHORT).show()
                pairedDeviceList() // Just refresh instead of restarting activity
            } else {
                //Toast.makeText(this, "Bluetooth enabling has been cancelled", Toast.LENGTH_SHORT).show()
            }
        }
    }


    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode == REQUEST_PERMISSIONS) {
            if (grantResults.isNotEmpty() && grantResults.all { it == PackageManager.PERMISSION_GRANTED }) {
                pairedDeviceList()
            } else {
                //Toast.makeText(this, "Permissions denied", Toast.LENGTH_SHORT).show()
            }
        }
    }
}


