package com.kedda.carApp


import android.annotation.SuppressLint
import android.app.Activity
import android.bluetooth.BluetoothAdapter
import android.content.Intent
import android.os.Bundle
import android.speech.RecognizerIntent
import android.util.Log
import android.widget.ImageButton
import android.widget.Switch
import android.widget.TextView
import android.widget.Toast
import java.io.IOException
import java.util.Locale


import android.os.Build
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.annotation.RequiresApi
import androidx.lifecycle.lifecycleScope
import com.kedda.carApp.ControlActivity.Companion.m_bluetoothAdapter
import kotlinx.coroutines.launch


class VoiceControlActivity : MainActivity() {

    companion object {

        lateinit var m_address: String
    }
    private lateinit var speechRecognitionLauncher: ActivityResultLauncher<Intent>

    @RequiresApi(Build.VERSION_CODES.ECLAIR)
    @SuppressLint("UseSwitchCompatOrMaterialCode")
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.voice_recognizer)
        m_address = intent.getStringExtra(EXTRA_ADDRESS).toString()

        m_bluetoothAdapter = BluetoothAdapter.getDefaultAdapter()


        val switch_layout = findViewById<Switch>(R.id.switch1)
        switch_layout.isChecked = true
        // Switch to ControlActivity when switched off

        // Handle switch toggle
        switch_layout.setOnCheckedChangeListener { _, isChecked ->


            if (!isChecked) {
                // Switch to button control mode
                lifecycleScope.launch {
                    BluetoothManager.sendCommand("ButtonModeAccessed\n")
                }
                  //  Toast.makeText(this, "Switching to voice Button Mode", Toast.LENGTH_SHORT).show()
                if (!isFinishing) {  // Check if activity is not finishing
                    val intent = Intent(this, ControlActivity::class.java)
                    intent.putExtra(EXTRA_ADDRESS, m_address)
                    startActivity(intent)
                    finish()  // Close the current activity
                }

            }

        }

        // Handle voice button click event
        val voiceButton = findViewById<ImageButton>(R.id.voiceButton)
        voiceButton.setOnClickListener {
            speak();
        }

        // Initialize speech recognition launcher and send the commands to the device
        speechRecognitionLauncher = registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
            if (result.resultCode == Activity.RESULT_OK) {
                try{
                    val resultData = result.data?.getStringArrayListExtra(RecognizerIntent.EXTRA_RESULTS)
                    resultData?.get(0)?.let { recognizedText ->
                        val speechText = findViewById<TextView>(R.id.instructionText)
                        speechText.text = recognizedText
                        // Send the recognized text as a command
                        lifecycleScope.launch {
                            BluetoothManager.sendCommand(recognizedText)
                        }

                    }
                }catch (e: IOException){
                    Log.e("voice command line 91", "command not sent :(", e)

                }

            }
        }
    }
    // speak function to start speech recognition launcher and send the commands to the device
    private fun speak() {
        val mIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
        mIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL,
            RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
        mIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE,
            Locale.getDefault())
        mIntent.putExtra(RecognizerIntent.EXTRA_PROMPT, "Hi, speak something")
        speechRecognitionLauncher.launch(mIntent)
    }



}